package de.intevation.flys.client.server;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import de.intevation.artifacts.httpclient.exceptions.ConnectionException;
import de.intevation.artifacts.httpclient.http.HttpClient;
import de.intevation.artifacts.httpclient.http.HttpClientImpl;
import de.intevation.artifacts.httpclient.http.response.DocumentResponseHandler;

import de.intevation.artifacts.common.ArtifactNamespaceContext;
import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.flys.client.shared.exceptions.ServerException;
import de.intevation.flys.client.client.services.CollectionItemAttributeService;

import de.intevation.flys.client.shared.model.Collection;
import de.intevation.flys.client.shared.model.Artifact;
import de.intevation.flys.client.shared.model.CollectionItemAttribute;
import de.intevation.flys.client.shared.model.Style;
import de.intevation.flys.client.shared.model.StyleSetting;


/**
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class CollectionItemAttributeServiceImpl
extends      RemoteServiceServlet
implements   CollectionItemAttributeService
{
    public static final String XPATH_RESULT = "/art:result/text()";

    public static final String OPERATION_FAILURE = "FAILED";


    public CollectionItemAttribute getCollectionItemAttribute(
        Collection collection,
        String artifact,
        String url,
        String locale)
    throws ServerException
    {
        System.out.println(
            "GetCollectionItemAttributeServiceImpl.getCollectionItemAttribute");

        Document requestDoc = XMLUtils.newDocument();

        XMLUtils.ElementCreator ec = new XMLUtils.ElementCreator(
                requestDoc,
                ArtifactNamespaceContext.NAMESPACE_URI,
                ArtifactNamespaceContext.NAMESPACE_PREFIX);

        Element action = ec.create("action");

        Element type = ec.create("type");
        ec.addAttr(type, "name", "getitemattribute", false);

        Element art = ec.create("artifact");
        ec.addAttr(art, "uuid", artifact, false);

        type.appendChild(art);
        action.appendChild(type);
        requestDoc.appendChild (action);

        try {
            HttpClient client = new HttpClientImpl(url, locale);
            Document res = (Document) client.doCollectionAction(
                requestDoc,
                collection.identifier(),
                new DocumentResponseHandler());

            return readXML (res);
        }
        catch (ConnectionException ce) {
            System.err.println(ce.getLocalizedMessage());
        }
        throw new ServerException("");
    }

    protected CollectionItemAttribute readXML (Document doc) {
        CollectionItemAttribute cia = new CollectionItemAttribute();

        Element root = doc.getDocumentElement();
        NodeList themes = root.getElementsByTagName("art:themes");

        if (themes.getLength() != 1) {
            return null;
        }

        Element e = (Element) themes.item(0);
        NodeList items = e.getElementsByTagName("theme");

        for (int i = 0; i < items.getLength(); i++) {
            cia.appendStyle(getStyle ((Element) items.item(i)));
        }

        return cia;
    }


    protected Style getStyle (Element element) {

        if (!element.getTagName().equals("theme")) {
            return null;
        }

        NodeList list = element.getElementsByTagName("field");
        Style style = new Style();
        style.setName (element.getAttribute("name"));
        for(int i = 0; i < list.getLength(); i++) {
            Element e = (Element) list.item(i);
            StyleSetting set = new StyleSetting (
                e.getAttribute("name"),
                e.getAttribute("default"),
                e.getAttribute("display"),
                e.getAttribute("hints"),
                e.getAttribute("type"));
            style.appendStyleSetting(set);
        }
        return style;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
